package com.mylar.sample.file.ui;

import java.io.UnsupportedEncodingException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;

import org.eclipse.mylar.tasks.ui.search.AbstractRepositoryQueryPage;
import org.eclipse.mylar.tasks.core.TaskRepository;
import org.eclipse.mylar.tasks.ui.TasksUiPlugin;

import com.mylar.sample.file.FileRepositoryQuery;

/**
 * This class is created to allow the user to entered information required
 * to query a repository.	The instance is created by 
 * FileRepositoryQueryWizard and added as a wizard page.
 */
public class FileRepositorySearchPage extends AbstractRepositoryQueryPage
{
	private static final String TITLE_SEARCH = "Sample File Respository Search Page";
	private static final String DESCRIPTION = "Select the file query parameters.";
	private static final String MAX_HITS_DEFAULT = "1000";
	
	private FileRepositoryQuery originalQuery = null;
	
	protected Text searchKey;
	protected Text maxHitsText;
	protected Text queryName;
	
	/**
	 * Class constuctor.  Performs basic initialization for the class.
	 * 
	 * @param repository	The repository to be queried.
	 */
	public FileRepositorySearchPage()
	{
		super(TITLE_SEARCH);
		setDescription(DESCRIPTION);
	}

	/**
	 * Class constuctor.  Performs basic initialization for the class.
	 * 
	 * @param repository	The repository to be queried.
	 */
	public FileRepositorySearchPage(TaskRepository repository)
	{
		super(TITLE_SEARCH);
		this.repository = repository;
		setDescription(DESCRIPTION);
	}

	/**
	 * Create display of fields that can be queried by the user.
	 */
	public void createControl(Composite parent)
	{
		Composite control = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout(2, false);
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		control.setLayout(layout);
		GridData gd = new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL);
		control.setLayoutData(gd);

		//
		//	Create query name
		//
		Label label = new Label(control, SWT.LEFT);
		label.setText("  Query Name ");
		queryName = new Text(control, SWT.BORDER);
		queryName.setTextLimit(64);
		
		//
		//	Create key information
		//
		label = new Label(control, SWT.LEFT);
		label.setText("  Key ");
		searchKey = new Text(control, SWT.BORDER);
		searchKey.setTextLimit(6);
		
		//
		//	Create max hit information
		//
		label = new Label(control, SWT.LEFT);
		label.setText("  Show a maximum of ");
		maxHitsText = new Text(control, SWT.BORDER);
		maxHitsText.setTextLimit(6);
		maxHitsText.setText(MAX_HITS_DEFAULT);
		
		setControl(control);		
	}
	
	/**
	 *	Get the query
	 *
	 *	@return	An instance of 'FileRepositoryQuery' that represents the query
	 *			to be executed.
	 */
	public FileRepositoryQuery getQuery()
	{
		if (originalQuery == null)
		{
			try
			{
				originalQuery = new FileRepositoryQuery(repository.getUrl(), getQueryURL(repository,
						getQueryParameters()), getQueryName(), getMaxHits(), TasksUiPlugin.getTaskListManager()
						.getTaskList());
			}
			catch (Exception e)
			{
				return null;
			}

		}
		else
		{
			try
			{
				originalQuery.setUrl(getQueryURL(repository, getQueryParameters()));
				originalQuery.setMaxHits(Integer.parseInt(getMaxHits()));
				originalQuery.setDescription(getQueryTitle());
			}
			catch (Exception e)
			{
				return null;
			}
		}
		return originalQuery;
	}

	/**
	 * Get the max number of hits to be returned by the query.
	 * 
	 * @return	The max number of hits to be return by the query.
	 */
	private String getMaxHits()
	{
		if (maxHitsText != null)
		{
			return maxHitsText.getText();
		}
		else return MAX_HITS_DEFAULT;
	}
	
	/**
	 * Creates a string of query parameters, based on information entered
	 * by the user.
	 * 
	 * @return	The query parameters entered by the user.
	 */
	private StringBuffer getQueryParameters()
	{
		StringBuffer sb = new StringBuffer();

		sb.append("key=");
		sb.append(searchKey.getText());
		
		return sb;
	}
	
	/**
	 * Get name of query entered by the user.
	 * 
	 * @return	The name of thge query entered by the user.
	 */
	private String getQueryName()
	{
		if (queryName != null)
		{
			return queryName.getText();
		}
		else
		{
			return "<Unknown>";
		}
	}
	
	/**
	 * Add the query parameters tp the query URL
	 * 
	 * @param repository	Repository to be queried.
	 * 
	 * @param params	Parameters for the query.
	 * 
	 * @return	A 'query' url.  The repository url and the query parameters.
	 */
	private String getQueryURL(TaskRepository repository, StringBuffer params)
	{
		StringBuffer url = new StringBuffer(getQueryURLStart(repository).toString());
		url.append(params);
		return url.toString();
	}

	/**
	 * Create start of query information 
	 * 
	 * @param repository	Repository to be queried.
	 * 
	 * @return	An instance of StringBuffer that contains the start of the query information
	 */
	private StringBuffer getQueryURLStart(TaskRepository repository)
	{
		StringBuffer sb = new StringBuffer(repository.getUrl());

		if (sb.charAt(sb.length() - 1) != '/') {
			sb.append('/');
		}
		sb.append("?");

		return sb;
	}
	
	public boolean canFlipToNextPage() {
		return false;
	}

}
